# OneRoster CSV サンプルコード

## 環境

* JavaScript

## サンプルコード

ZIPファイル選択時にイベントonZipFileSpecified()を実行し、
正常であれば登録API処理を、エラーの場合はエラーメッセージを表示する(コード内では独自処理のため具体的な内容は割愛)。

```

/**
* ZIPファイル選択イベント
* @param e Eventオブジェクト
*/
onZipFileSpecified(e) {
    const file = e.target.files[0];
    const fileName = file.name;

    // ZIPファイルを読込み(自社独自領域のため詳細は割愛)
    // ZIPファイルのうち、users.csvのみ対象としCSVファイルをパースしたあと確認メソッドを呼び出す。
    if (zipFilesName.indexOf('users.csv') > -1) {
      // results=CSVパース(自社独自領域のため詳細は割愛)
      // 検証メソッドを呼び出す
      this.checkCSV(results, fileName);
    }
},
/**
* users.csvの内容検証
* @param results users.csvの読込み値(Json)
* @param fileName 選択時Zipファイル名
*/
checkCSV(results, fileName) {
  // ヘッダーリスト
  const headerList = ['sourcedId', 'status', 'dateLastModified', 'enabledUser', 'username', 'userIds', 'givenName', 'familyName', 'middleName', 'identifier', 'email', 'sms', 'phone', 'agentSourcedIds', 'grades', 'password', 'userMasterIdentifier', 'resourceSourcedIds', 'preferredGivenName', 'preferredMiddleName', 'preferredFamilyName', 'primaryOrgSourcedId', 'pronouns', 'metadata.jp.kanaGivenName', 'metadata.jp.kanaFamilyName', 'metadata.jp.kanaMiddleName', 'metadata.jp.homeClass'];
  for (var i = 0; i < headerList.length; i++) {
    if (headerList[i] !== results.meta.fields[i]) {
      // ヘッダー不正のためエラーメッセージ表示(自社独自領域のため詳細は割愛)
      return;
    }
  }

  // 必須値リスト
  const requiredList = ['sourcedId', 'enabledUser', 'username', 'givenName', 'familyName', 'userMasterIdenfitier', 'preferredGivenName', 'preferredFamilyName', 'metadata.jp.kanaGivenName', 'metadata.jp.kanaFamilyName', 'metadata.jp.homeClass'];

  const currentYear; // 現在年度の算出(自社独自領域のため詳細は割愛)

  var students = [];
  var errorList = [];
  var row = 0;
  // CSVデータ行の各値をチェックする
  for (var rowData of results.data) {
    row++;

    var errId = [];

    // 必須項目の値チェック。エラーの項目をリストに追加する。
    for (var title of requiredList) {
      if (!rowData[title]) {
        errId.push(title);
      }
    }

    if (errId.length === 0) {
      // 対象項目の各データ形式チェック。エラーの項目をリストに追加する。
      if (!this.isUUID(rowData.sourcedId)) {
        errId.push('sourcedId');
      }
      if (rowData.enabledUser !== 'true') {
        errId.push('enabledUser');
      }
      if (this.checkContainHanKatakana(rowData.givenName)) {
        errId.push('givenName');
      }
      if (this.checkContainHanKatakana(rowData.familyName)) {
        errId.push('familyName');
      }
      if (rowData.phone && !rowData.phone.match(/^(\+)*[-()0-9]+$/)) {
        errId.push('phone');
      }
      if (!this.isUUIDV4(rowData.userMasterIdentifier)) {
        errId.push('userMasterIdenfitier');
      }
      if (rowData.grades && !this.getEnterYearByGradeCode(rowData.grades, currentYear)) {
        errId.push('grades');
      }

      if (this.checkContainHanKatakana(rowData.preferredGivenName)) {
        errId.push('preferredGivenName');
      }
      if (this.checkContainHanKatakana(rowData.preferredFamilyName)) {
        errId.push('preferredFamilyName');
      }
      if (this.checkContainHanKatakana(rowData['metadata.jp.kanaGivenName'])) {
        errId.push('metadata.jp.kanaGivenName');
      }
      if (this.checkContainHanKatakana(rowData['metadata.jp.kanaFamilyName'])) {
        errId.push('metadata.jp.kanaFamilyName');
      }
      if (!this.isUUID(rowData['metadata.jp.homeClass'])) {
        errId.push('metadata.jp.homeClass');
      }
    }

    if (0 < errId.length) {
      // 項目エラーIDがある場合は、エラー保持リストに追加(自社独自領域のため詳細は割愛)
      errorList.push({});
      continue;
    }

    // CSVの値をAPIに渡すリストに追加(自社独自領域のため詳細は割愛)
    students.push({});
  }

  if (0 < errorList.length) {
    // ファイル内容の項目エラー表示(自社独自領域のため詳細は割愛)
  } else if (0 < students.length) {
    // APIの呼び出し(自社独自領域のため詳細は割愛)
  } else {
    // ファイル不正エラー表示(自社独自領域のため詳細は割愛)
  }
},
/**
* 半角カナを含むか
* @param str 文字列
*/
checkContainHanKatakana(str) {
  // 半角カタカナを含むか
  if (str && str.trim().match(/[ｱ-ﾝﾞﾟ]+/)) {
    return true;
  }
  return false;
},
/**
* UUID形式か
* @param str 文字列
*/
isUUID(str) {
  // UUID形式か
  if (str && str.toLowerCase().match(/([0-9a-f]{8})-([0-9a-f]{4})-([0-9a-f]{4})-([0-9a-f]{4})-([0-9a-f]{12})/)) {
    return true;
  }
  return false;
},
/**
* UUID V4形式か
* @param str 文字列
*/
isUUIDV4(str) {
  // UUID(v4)形式か
  if (str && str.toLowerCase().match(/([0-9a-f]{8})-([0-9a-f]{4})-4([0-9a-f]{3})-([0-9a-f]{4})-([0-9a-f]{12})/)) {
    return true;
  }
  return false;
},
/**
* 学年から入学年度を取得する。
* @param gradeCode 学年コード値
* @param currentYear 現在年度
*/
getEnterYearByGradeCode(gradeCode, currentYear) {
  // 入学年度は独自システムで使用する
  // 返却が空の場合は学年の値が不一致のためエラーとする
  var diffYear = 0;
  if (gradeCode !== '') {
    if (gradeCode.match('J3')) {
      diffYear = 2;
    } else if (gradeCode.match('J2')) {
      diffYear = 1;
    } else if (gradeCode.match('J1')) {
      //中1 現在年度
    } else if (gradeCode.match('P6')) {
      diffYear = 5;
    } else if (gradeCode.match('P5')) {
      diffYear = 4;
    } else if (gradeCode.match('P4')) {
      diffYear = 3;
    } else if (gradeCode.match('P3')) {
      diffYear = 2;
    } else if (gradeCode.match('P2')) {
      diffYear = 1;
    } else if (gradeCode.match('P1')) {
      //小1 現在年度
    } else {
      // 規定外エラー
      return '';
    }
  }
  return (currentYear - diffYear).toString();
}

```
